#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Aug 21
Ce script trace les périodes de lever des astres,
un jour par mois.
Heures : UTC
Liste des astres modifiables.
@author: David Alberto (astrolabe-science.fr)
"""

from skyfield import almanac # recherche d'heures de lever/coucher
from skyfield.api import N, W, S, E, wgs84, load, utc
import matplotlib.pyplot as plt # graphiques
import datetime as dt # création et gestion de dates/heures
import locale # noms des mois en français

# Réglages utilisateur :------------------------
locale.setlocale(locale.LC_ALL, 'fr_FR.UTF-8')# mois en français
plt.rcParams["font.family"] = "Roboto" # ou autre police installée

annee   = 2023
mois    = 9
latitude = 49.5 * N
longitude = 0.11 * E
altitude = 20 # en mètres
lieu_observation = wgs84.latlon(latitude, longitude, elevation_m = altitude)
nom_lieu = "Le Havre"
NomFichier = "VisibilitePlanetesMensuelSept23"
decalage_nom = -0.23# décalage vertical du nom des astres par rapport à la ligne (négatif : vers le haut)
# styles :
prop_lignes=dict(lw=4, solid_capstyle='butt')
prop_texte = dict(fontweight='normal',fontsize=8,va='center')
prop_heures = dict(fontweight='normal',fontsize=6, va='center')
ncols = 6 # nombre de colonnes dans la grille d'axes.

# durées pour définir les limites des fenêtres (durée avant le coucher et après le lever du Soleil).
duree2H = dt.timedelta(hours=2)# fenêtre graphique 2h avant le coucher et 2h après le jour.
dureecrep = dt.timedelta(hours=2)# pour durée du crépuscule
#------------------------------------------------------------------------------
# chargement du fichier d'éphémérides :
# voir la docu du package Skyfield pour d'autres périodes.
eph = load('../skyfield-ephemerides/de421.bsp')# charger fichier d'éphémérides (1900-2050)
# eph = load('../skyfield-ephemerides/de440s.bsp')# charger fichier d'éphémérides (1849-2150)
soleil= eph['sun']
mercure = eph['mercury']
venus = eph['venus']
terre= eph['earth']
mars = eph['mars']
jupiter = eph['jupiter barycenter']
saturne = eph['saturn barycenter']
# uranus = eph['uranus barycenter']
# neptune = eph['neptune barycenter']
# pluton = eph['pluto barycenter']
lune = eph['moon']

# création des caractéristiques des astres -------------------------
# ces 3 listes doivent avoir la même longueur.
liste_planetes=[mercure, venus, mars, jupiter, saturne, lune]
liste_couleurs = ['darkgray', 'orange', 'red', 'thistle', 'tan','gray']
liste_noms = ['Mercure', 'Vénus', "Mars", "Jupiter", "Saturne", "Lune"]

#------------------------------------------------------------------------------
ts = load.timescale() # crée une échelle de dates Skyfield

def Njours(mois):
    """
    renvoie le nombre de jours dans le mois choisi.
    """
    if mois == 12:
        N = 31
    else:
        duree = dt.datetime(annee, mois+1,1) - dt.datetime(annee, mois,1)
        N = duree.days
    return N

def CoucherLeverSoleil(J):
    """
    J : rang du jour du mois.
    Renvoie les heures de coucher (de la veille) et du lever du jour,
    en format datetime.
    """
    veille = ts.utc(annee,mois, J-1,0)
    jour = ts.utc(annee,mois, J,0)
    f = almanac.risings_and_settings(eph, soleil, lieu_observation)
    t, y = almanac.find_discrete(veille, jour, f)
    _, Hcoucher = t
    demain = ts.utc(annee,mois, J+1,0)
    f = almanac.risings_and_settings(eph, soleil, lieu_observation)
    t, y = almanac.find_discrete(jour, demain, f)
    Hlever, _ = t
    return Hcoucher.utc_datetime(), Hlever.utc_datetime()

def LeversCouchers(J,astre):
    """
    calcule 4 dates : levers et couchers de la veille et du jour.
    """
    veille = ts.utc(annee,mois,J-1)
    jour = ts.utc(annee,mois,J)
    demain = ts.utc(annee,mois,J+1)
    # recherche des heures de lever/coucher de la veille :
    # t contient une ou plusieurs valeurs de dates,
    # y contient une série de valeurs : 0 si coucher, 1 si lever
    f = almanac.risings_and_settings(eph, astre, lieu_observation)
    t, y = almanac.find_discrete(veille, jour, f)
    datea = t[0]
    ya = y[0]
    if len(t) == 1: # si une seule date trouvée
        dateb = veille
    else:
        dateb = t[1]
    if ya: # si datea est un lever
        L1 = datea
        C1 = dateb
    else:
        L1 = dateb
        C1 = datea
    t, y = almanac.find_discrete(jour, demain, f)
    datea = t[0]
    ya = y[0]
    if len(t) == 1:
        dateb = demain
    else:
        dateb = t[1]
    if ya:
        L2 = datea
        C2 = dateb
    else:
        L2 = dateb
        C2 = datea
    return L1.utc_datetime(), C1.utc_datetime(), L2.utc_datetime(), C2.utc_datetime()
    
def heures_lim(J):
    """
    J : rang du jour dans le mois.
    Renvoie les heures limites pour l'axe : une meme durée avant le coucher et après le lever du Soleil.
    """
    heure_debut,_ = CoucherLeverSoleil(J)
    _, heure_fin = CoucherLeverSoleil(J)
    heure_debut = heure_debut - duree2H
    heure_fin = heure_fin + duree2H
    return heure_debut, heure_fin

def format_axe(axe, J):
    """
    met en forme l'axe (valeurs limites, rectangles colorés, échelle de temps)
    J est le rang du jour.
    """
    axe.set_facecolor("midnightblue")
    axe.invert_yaxis() # inversion du sens de l'axe y
    axe.set_ylim(len(liste_planetes)+1,0)
    axe.set_xlim(xmin, xmax)
    axe.set_yticks([])
    axe.set_xticks([coucherS, leverS])
    axe.set_xticklabels([arrondi_heure(coucherS),arrondi_heure(leverS)], fontsize=8)
    axe.set_title(dt.datetime(annee, mois, J).strftime('%d %B'), fontsize=10)
    rectangle_nuit = plt.Rectangle((coucherS,0), leverS - coucherS, 10, facecolor='k')
    axe.add_patch(rectangle_nuit)
    crepuscule = plt.Rectangle((coucherS,0), dureecrep, 10, facecolor='midnightblue',alpha=0.5)
    axe.add_patch(crepuscule)
    aube = plt.Rectangle((leverS-dureecrep,0), dureecrep, 10, facecolor='midnightblue',alpha=0.5)
    axe.add_patch(aube)

def ligne(astre, J):
    """
    Trace la ligne de passage au-dessus de l'horizon, avec les heures et nom de l'astre.
    """
    index = liste_planetes.index(astre)
    nom = liste_noms[index]
    couleur = liste_couleurs[index]
    y = index + 1
    L1, C1, L2, C2 = LeversCouchers(J, astre)
    # tracé des lignes :
    if L1 > C1:
        axe.plot([L1,C2],[y,y],color = couleur,**prop_lignes)
    else:
        axe.plot([L1,C1],[y,y],color = couleur,**prop_lignes)
        axe.plot([L2,C2],[y,y],color = couleur,**prop_lignes)
        
        # affichage des heures :
    if L1 >= xmin and L1 <= xmax:
        axe.text(L1,y,arrondi_heure(L1),
             color=couleur,ha='right',**prop_heures)
    if L2 >= xmin and L2 <= xmax:
        axe.text(L2,y,arrondi_heure(L2),
             color=couleur,ha='right',**prop_heures)
    if C1 > xmin and C1 <= xmax:
        axe.text(C1,y,arrondi_heure(C1),
             color=couleur,ha='left',**prop_heures)
    if C2 > xmin and C2 <= xmax:
        axe.text(C2,y,arrondi_heure(C2),
             color=couleur,ha='left',**prop_heures)
    # inscription du nom des astres :
    axe.text(xmin,y + decalage_nom,nom,ha='left',
             c=couleur,**prop_texte)

def arrondi_heure(heure):
    """
    depuis une heure format datetime,
    renvoie une heure arrondie à la minute près.
    """
    exces = dt.timedelta(seconds=heure.second,
                         microseconds = heure.microsecond)
    
    heure = heure - exces
    if exces >= dt.timedelta(seconds=30):
        heure += dt.timedelta(seconds=60)
    return heure.strftime('%H:%M')

def Nlignes(mois):
    """
    Renvoie le nombre de lignes dans la grille, selon le nombre de jours du mois.
    """
    if nb_jours % ncols == 0:
        lignes = int(nb_jours / ncols)
    else:
        lignes = int( nb_jours /ncols) + 1
    return lignes

# ####################################################"
# création du graphique :
nb_jours = Njours(mois)
nlig = Nlignes(mois)

fig,axs = plt.subplots(ncols=ncols,nrows=nlig
                        ,tight_layout=True
                       )
# fig.set_size_inches(42 / 2.54, 29.7 / 2.54) # format A3 paysage
fig.set_size_inches(29.7 / 2.54, 42 / 2.54) # format A3 portrait
axs = axs.flatten() # pour rendre itérables les axes.

J1 = dt.datetime(annee,mois,1)
fig.suptitle(f"Lever et coucher des astres en {J1.strftime('%B %Y')}",
              fontsize=16, fontweight='bold',y = 0.986)


for J in range(1, ncols * nlig + 1):
    axe = axs[J-1]
    if J > nb_jours:
        axe.axis('off')
    else:
        xmin, xmax = heures_lim(J)
        coucherS, leverS = CoucherLeverSoleil(J)
        format_axe(axe, J)
        for planete in liste_planetes:
        # for planete in [lune]:# pour limiter la liste d'astres
            ligne(planete, J)

fig.subplots_adjust(left=0.01,
                    right=0.2 # extra marge à droite
                    # ,wspace = 0
                    )

# légendes :
texte = f"heure TU, lieu d'observation : {nom_lieu}, lat. {latitude}°, lon. {longitude}°.\t La nuit commence la veille de la date indiquée. \t\t\t\t\t D. Alberto (www.astrolabe-science.fr)"
fig.text(0.985,0.010,texte,fontsize=8,va='bottom',rotation=90,
            transform = fig.transFigure)

# plt.show()
fig.savefig(NomFichier + ".pdf")
fig.savefig(NomFichier + ".png",dpi=200)
